@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2011-2013 Colin Jones
- * Copyright (C) 2014-2019 Valère Monseur
+ * Copyright (C) 2014-2022 Valère Monseur
  *
  * Based on code by Matteo Marchesotti
  * Copyright (C) 2007 Matteo Marchesotti <matteo.marchesotti@fsfe.org>
@@ -21,8 +21,8 @@
  * with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#define CBATTICON_VERSION_NUMBER 1.6.9
-#define CBATTICON_VERSION_STRING "1.6.9"
+#define CBATTICON_VERSION_NUMBER 1.6.13
+#define CBATTICON_VERSION_STRING "1.6.13"
 #define CBATTICON_STRING         "cbatticon"
 
 #include <glib.h>
@@ -54,6 +54,7 @@ static gboolean get_battery_status (gint *status);
 
 static gboolean get_battery_full_capacity (gboolean *use_charge, gdouble *capacity);
 static gboolean get_battery_remaining_capacity (gboolean use_charge, gdouble *capacity);
+static gboolean get_battery_remaining_capacity_pct (gdouble *capacity);
 static gboolean get_battery_current_rate (gboolean use_charge, gdouble *rate);
 
 static gboolean get_battery_charge (gboolean remaining, gint *percentage, gint *time);
@@ -110,6 +111,7 @@ struct configuration {
     gint     icon_type;
     gint     low_level;
     gint     critical_level;
+    gchar   *command_low_level;
     gchar   *command_critical_level;
     gchar   *command_left_click;
 #ifdef WITH_NOTIFY
@@ -164,6 +166,7 @@ static gint get_options (int argc, char **argv)
         { "icon-type"             , 'i', 0, G_OPTION_ARG_STRING, &icon_type_string                    , N_("Set icon type ('standard', 'notification' or 'symbolic')") , NULL },
         { "low-level"             , 'l', 0, G_OPTION_ARG_INT   , &configuration.low_level             , N_("Set low battery level (in percent)")                       , NULL },
         { "critical-level"        , 'r', 0, G_OPTION_ARG_INT   , &configuration.critical_level        , N_("Set critical battery level (in percent)")                  , NULL },
+        { "command-low-level"     , 'o', 0, G_OPTION_ARG_STRING, &configuration.command_low_level     , N_("Command to execute when low battery level is reached")     , NULL },
         { "command-critical-level", 'c', 0, G_OPTION_ARG_STRING, &configuration.command_critical_level, N_("Command to execute when critical battery level is reached"), NULL },
         { "command-left-click"    , 'x', 0, G_OPTION_ARG_STRING, &configuration.command_left_click    , N_("Command to execute when left clicking on tray icon")       , NULL },
 #ifdef WITH_NOTIFY
@@ -590,6 +593,13 @@ static gboolean get_battery_remaining_capacity (gboolean use_charge, gdouble *ca
     }
 }
 
+static gboolean get_battery_remaining_capacity_pct (gdouble *capacity)
+{
+    g_return_val_if_fail (capacity != NULL, FALSE);
+
+    return get_sysattr_double (battery_path, "capacity", capacity);
+}
+
 static gboolean get_battery_current_rate (gboolean use_charge, gdouble *rate)
 {
     if (use_charge == FALSE) {
@@ -619,11 +629,16 @@ static gboolean get_battery_charge (gboolean remaining, gint *percentage, gint *
     }
 
     if (get_battery_remaining_capacity (use_charge, &remaining_capacity) == FALSE) {
-        if (configuration.debug_output == TRUE) {
-            g_printf ("remaining capacity: %s\n", "unavailable");
+        if (get_battery_remaining_capacity_pct (&remaining_capacity) == FALSE) {
+            if (configuration.debug_output == TRUE) {
+                g_printf ("remaining capacity: %s\n", "unavailable");
+            }
+
+            return FALSE;
         }
 
-        return FALSE;
+        /* remaining capacity is percentage, compute the actual remaining capacity */
+        remaining_capacity *= full_capacity / 100.0;
     }
 
     *percentage = (gint)fmin (floor (remaining_capacity / full_capacity * 100.0), 100.0);
@@ -735,6 +750,7 @@ static void update_tray_icon_status (GtkStatusIcon *tray_icon)
     static gboolean ac_only                = FALSE;
     static gboolean battery_low            = FALSE;
     static gboolean battery_critical       = FALSE;
+    static gboolean spawn_command_low      = FALSE;
     static gboolean spawn_command_critical = FALSE;
 
     gint percentage, time;
@@ -755,6 +771,7 @@ static void update_tray_icon_status (GtkStatusIcon *tray_icon)
         ac_only                = FALSE;
         battery_low            = FALSE;
         battery_critical       = FALSE;
+        spawn_command_low      = FALSE;
         spawn_command_critical = FALSE;
     }
 
@@ -861,6 +878,7 @@ static void update_tray_icon_status (GtkStatusIcon *tray_icon)
 
                 battery_low            = FALSE;
                 battery_critical       = FALSE;
+                spawn_command_low      = FALSE;
                 spawn_command_critical = FALSE;
             }
 
@@ -869,6 +887,8 @@ static void update_tray_icon_status (GtkStatusIcon *tray_icon)
 
                 battery_string = get_battery_string (LOW_LEVEL, percentage);
                 NOTIFY_MESSAGE (&notification, battery_string, time_string, NOTIFY_EXPIRES_NEVER, NOTIFY_URGENCY_NORMAL);
+
+                spawn_command_low = TRUE;
             }
 
             if (battery_critical == FALSE && percentage <= configuration.critical_level) {
@@ -883,6 +903,34 @@ static void update_tray_icon_status (GtkStatusIcon *tray_icon)
             gtk_status_icon_set_tooltip_text (tray_icon, get_tooltip_string (battery_string, time_string));
             gtk_status_icon_set_from_icon_name (tray_icon, get_icon_name (battery_status, percentage));
 
+            if (spawn_command_low == TRUE) {
+                spawn_command_low = FALSE;
+
+                if (configuration.command_low_level != NULL) {
+                    syslog (LOG_CRIT, _("Spawning low battery level command in 5 seconds: %s"), configuration.command_low_level);
+                    g_usleep (G_USEC_PER_SEC * 5);
+
+                    if (get_battery_status (&battery_status) == TRUE) {
+                        if (battery_status != DISCHARGING && battery_status != NOT_CHARGING) {
+                            syslog (LOG_NOTICE, _("Skipping low battery level command, no longer discharging"));
+                            return;
+                        }
+                    }
+
+                    if (g_spawn_command_line_async (configuration.command_low_level, &error) == FALSE) {
+                        syslog (LOG_CRIT, _("Cannot spawn low battery level command: %s\n"), error->message);
+
+                        g_printerr (_("Cannot spawn low battery level command: %s\n"), error->message);
+                        g_error_free (error); error = NULL;
+
+#ifdef WITH_NOTIFY
+                        static NotifyNotification *spawn_notification = NULL;
+                        NOTIFY_MESSAGE (&spawn_notification, _("Cannot spawn low battery level command!"), configuration.command_low_level, NOTIFY_EXPIRES_NEVER, NOTIFY_URGENCY_CRITICAL);
+#endif
+                    }
+                }
+            }
+
             if (spawn_command_critical == TRUE) {
                 spawn_command_critical = FALSE;
 
@@ -890,6 +938,13 @@ static void update_tray_icon_status (GtkStatusIcon *tray_icon)
                     syslog (LOG_CRIT, _("Spawning critical battery level command in 30 seconds: %s"), configuration.command_critical_level);
                     g_usleep (G_USEC_PER_SEC * 30);
 
+                    if (get_battery_status (&battery_status) == TRUE) {
+                        if (battery_status != DISCHARGING && battery_status != NOT_CHARGING) {
+                            syslog (LOG_NOTICE, _("Skipping critical battery level command, no longer discharging"));
+                            return;
+                        }
+                    }
+
                     if (g_spawn_command_line_async (configuration.command_critical_level, &error) == FALSE) {
                         syslog (LOG_CRIT, _("Cannot spawn critical battery level command: %s\n"), error->message);
 
